from agency_swarm.agents import Agent


class Copywriter(Agent):
    def __init__(self):
        super().__init__(
            name="Copywriter",
            description="Responsible for creating compelling content for the web applications. Collaborates with the Web Developer for content integration.",
            instructions="./instructions.md",
            # files_folder="./files",
            # schemas_folder="./schemas",
            tools=[],
            tools_folder="./tools"
        )



from agency_swarm import Agency, set_openai_key
from .CEO import CEO
from .Copywriter import Copywriter
from .WebDeveloper import WebDeveloper
from .Designer import Designer
import os

# Prompt the user for the OpenAI API key
def get_openai_key():
    return input("Please enter your OpenAI API key: ")

# Set the API key
set_openai_key(get_openai_key())

ceo = CEO()
designer = Designer()
web_developer = WebDeveloper()
copywriter = Copywriter()

agency = Agency([ceo, designer, web_developer,
                 [ceo, designer],
                 [designer, web_developer],
                 [designer, copywriter]],
                shared_instructions='./agency_manifesto.md')



def rundemo():
    agency.run_demo()

if __name__ == '__main__':
    # agency.demo_gradio(height=400)
    agency.run_demo()


from agency_swarm.agents import Agent
from .tools import AnalyzeContent

class Designer(Agent):
    def __init__(self):
        super().__init__(
            name="Designer",
            description="An agent that analyzes the current browser window for WebDevCrafters.",
            instructions="./instructions.md",
            # files_folder="./files",
            # schemas_folder="./schemas",
            tools=[AnalyzeContent],
            # tools_folder="./tools"
        )



from agency_swarm.agents import Agent


class CEO(Agent):
    def __init__(self):
        super().__init__(
            name="CEO",
            description="Acts as the overseer and communicator across the agency, ensuring alignment with the agency's goals.",
            instructions="./instructions.md",
            # files_folder="./files",
            # schemas_folder="./schemas",
            tools=[],
            tools_folder="./tools"
        )



from agency_swarm.tools import BaseTool
from pydantic import Field


class FileReader(BaseTool):
    """This tool reads a file and returns the contents along with line numbers on the left."""
    file_path: str = Field(
        ..., description="Path to the file to read with extension.",
        examples=["./file.txt", "./file.json", "../../file.py"]
    )

    def run(self):
        # read file
        with open(self.file_path, "r") as f:
            file_contents = f.readlines()

        # return file contents
        return "\n".join([f"{i + 1}. {line}" for i, line in enumerate(file_contents)])



import os
import subprocess
import time
from typing import Literal
from pydantic import Field
from agency_swarm.tools import BaseTool


class RunCommand(BaseTool):
    """Enables you to execute certain terminal commands."""
    command: Literal['build', 'create-next-app', 'install'] = Field(...,
                                                                    description='The terminal command to execute. Create a new Next.js app with `create-next-app`, build the current app with `build`, or install dependencies with `install`.'
                                                                    )
    options: str = Field(
        default='',
        description='Additional options to pass to the command. For example, you can pass package names to install with `npm install`.',
        examples=['--save', '--save-dev', 'my-next-app']
    )

    def run(self):
        try:
            options_list = [option for option in self.options.split(' ') if option]

            if self.command == 'create-next-app':
                if len(options_list) == 0:
                    return 'Please provide a name for the app when using the `create-next-app` command.'

                app_name = options_list[0]  # Extract app name from the options
                if self.shared_state.get('app_directory'):
                    return 'You have already created a Next.js app.'

                cmd = [
                    'npx', 'create-next-app', '--tailwind', app_name, '--use-npm', '--ts',
                    '--no-eslint', '--src-dir', '--no-app', '--import-alias', '@/*'
                ]

                process = subprocess.run(cmd, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

                if process.returncode == 0:
                    app_directory = os.path.join(os.getcwd(), app_name)
                    self.shared_state.set('app_directory', app_directory)
                    return f'Command executed successfully. App directory: {app_directory}\nOutput:\n{process.stdout} The application uses typescript.'
                else:
                    return f'Error executing command: {process.stderr}'

            # Ensures an app has been created before proceeding with 'build' or 'install'
            if not self.shared_state.get('app_directory'):
                return 'Please create a Next.js app first using the `create-next-app` command.'

            # Change to the app directory before running 'build' or 'install'
            current_directory = os.getcwd()
            os.chdir(self.shared_state.get('app_directory'))

            if self.command == 'build':
                cmd = ['npm', 'run', 'build'] + options_list
                process = subprocess.run(cmd, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

            elif self.command == 'install':
                # For 'npm install', options might include package names or flags
                cmd = ['npm', 'install'] + options_list
                process = subprocess.run(cmd, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

            else:
                return 'Invalid command. Please use either `create-next-app`, `build`, or `install`.'

            os.chdir(current_directory)  # Ensure the directory is switched back

            # Return the full output of 'build' or 'install'
            return f'Command executed successfully. Output:\n{process.stdout.decode()}'

        except subprocess.CalledProcessError as e:
            if self.command == 'build':
                return f'Error building the app: {e.stderr}.\n\nPlease make sure to read the problematic file with the FileReader tool and fix the errors. Rewrite the entire file with FileWriter tool if you get stuck.'
            return f'Error executing command: {e.stderr}'



from agency_swarm.tools import BaseTool
from pydantic import Field
import os


class DirectoryNavigator(BaseTool):
    """Allows the WebDeveloper agent to navigate directories."""
    path: str = Field(
        ..., description="The path of the directory to navigate to."
    )

    def run(self):
        try:
            os.chdir(self.path)
            return f'Successfully changed directory to: {self.path}'
        except Exception as e:
            return f'Error changing directory: {e}'



import os

from agency_swarm.tools import BaseTool
from pydantic import Field


class FileWriter(BaseTool):
    """Allows you to write new files or modify existing files."""
    chain_of_thought: str = Field(
        ..., description="Please think step-by-step about what needs to be written to the file in order for the program to match the requirements.",
        exclude=True,
    )
    file_path: str = Field(
        ..., description="The path of the file to write or modify. Will create directories if they don't exist."
    )
    content: str = Field(
        ..., description="The full content of the file to write. Content must not be truncated and must represent a correct "
                         "functioning program with all the imports defined."
    )

    def run(self):
        if not self.shared_state.get('app_directory'):
            return "You must create an app first to use this tool."

        if self.shared_state.get('app_directory') not in os.getcwd():
            return "You must be in the root directory of the app to use this tool."
        try:
            # create directories if they don't exist
            dir_path = os.path.dirname(self.file_path)
            os.makedirs(dir_path, exist_ok=True)

            with open(self.file_path, 'w') as file:
                file.write(self.content)
            return f'Successfully wrote to file: {self.file_path}. Please make sure to build the app next to ensure that there are no errors.'
        except Exception as e:
            return f'Error writing to file: {e}'



from agency_swarm.agents import Agent
from .tools import ChangeLines, CheckCurrentDir,DirectoryNavigator,RunCommand,FileWriter,FileReader,ListDir

class WebDeveloper(Agent):
    def __init__(self):
        super().__init__(
            name="WebDeveloper",
            description="A versatile agent for WebDevCrafters capable of navigating directories, reading, writing, modifying files, and executing terminal commands.",
            instructions="./instructions.md",
            # files_folder="./files",
            # schemas_folder="./schemas",
            tools=[ChangeLines, CheckCurrentDir, DirectoryNavigator, RunCommand, FileWriter, FileReader, ListDir],
            # tools_folder="./tools"
        )



import os
from typing import Literal, Optional, List

from instructor import OpenAISchema
from pydantic import Field, model_validator, field_validator

from agency_swarm import BaseTool


class LineChange(OpenAISchema):
    """
    Line changes to be made.
    """
    chain_of_thought: str = Field(
        ..., description="Please think step-by-step about the required changes to the file in order to construct a fully functioning and correct program.",
        exclude=True,
    )
    line_number: int = Field(
        ..., description="Line number to change.",
        examples=[1, 2, 3]
    )
    new_line: Optional[str] = Field(
        None, description="New line to replace the old line. Not required only for delete mode.",
        examples=["This is a new line"]
    )
    mode: Literal["replace", "insert", "delete"] = Field(
        "replace", description='Mode to use for the line change. "replace" replaces the line with the new line, '
                               '"insert" inserts the new line at the specified line number, and "delete" deletes the '
                               "specified line number.",
    )

    @model_validator(mode='after')
    def validate_new_line(self):
        mode, new_line = self.mode, self.new_line
        if mode == "delete" and new_line is not None:
            raise ValueError("new_line should not be specified for delete mode.")
        elif mode in ["replace", "insert"] and new_line is None:
            raise ValueError("new_line should be specified for replace and insert modes.")
        return self


class ChangeLines(BaseTool):
    """
    This tool changes specified lines in a file. Returns the new file contents.
    """
    file_path: str = Field(
        ..., description="Path to the file with extension.",
        examples=["./file.txt", "./file.json", "../../file.py"]
    )
    changes: List[LineChange] = Field(
        ..., description="Line changes to be made to the file.",
        examples=[{"line_number": 1, "new_line": "This is a new line", "mode": "replace"}]
    )

    def run(self):
        if not self.shared_state.get('app_directory'):
            return "You must create an app first to use this tool."

        if self.shared_state.get('app_directory') not in os.getcwd():
            return "You must be in the root directory of the app to use this tool."

        # read file
        with open(self.file_path, "r") as f:
            file_contents = f.readlines()

            # Process changes in a way that accounts for modifications affecting line numbers
            for change in sorted(self.changes, key=lambda x: x.line_number, reverse=True):
                try:
                    if change.mode == "replace" and 0 < change.line_number <= len(file_contents):
                        file_contents[change.line_number - 1] = change.new_line + '\n'
                    elif change.mode == "insert":
                        file_contents.insert(change.line_number - 1, change.new_line + '\n')
                    elif change.mode == "delete" and 0 < change.line_number <= len(file_contents):
                        file_contents.pop(change.line_number - 1)
                except IndexError:
                    return f"Error: Line number {change.line_number} is out of the file's range."

        # write file
        with open(self.file_path, "w") as f:
            f.writelines(file_contents)

        with open(self.file_path, "r") as f:
            file_contents = f.readlines()

        # return file contents with line numbers
        return "\n".join([f"{i + 1}. {line}" for i, line in enumerate(file_contents)])

    # use field validation to ensure that the file path is valid
    @field_validator("file_path", mode='after')
    @classmethod
    def validate_file_path(cls, v: str):
        if not os.path.exists(v):
            raise ValueError("File path does not exist.")

        return v


from .ChangeLines import ChangeLines
from .CheckCurrentDir import CheckCurrentDir
from .DirectoryNavigator import DirectoryNavigator
from .RunCommand import RunCommand
from .FileWriter import FileWriter
from .FileReader import FileReader
from .ListDir import ListDir


from pydantic import Field

from agency_swarm import BaseTool
import os


class ListDir(BaseTool):
    """
    This tool returns the tree structure of the directory.
    """
    dir_path: str = Field(
        ..., description="Path of the directory to read.",
        examples=["./", "./test", "../../"]
    )

    def run(self):
        import os

        if not self.shared_state.get('app_directory'):
            return "You must create an app first to use this tool."

        if self.shared_state.get('app_directory') not in os.getcwd():
            return "You must be in the root directory of the app to use this tool."

        tree = []
        def list_directory_tree(path, indent=''):
            """Recursively list the contents of a directory in a tree-like format."""
            if not os.path.isdir(path):
                raise ValueError(f"The path {path} is not a valid directory")

            items = os.listdir(path)
            # exclude common hidden files and directories
            exclude = ['.git', '.idea', '__pycache__', 'node_modules',
                       '.DS_Store', '.vscode', '.next', 'dist', 'build', 'out']
            items = [item for item in items if item not in exclude]

            for i, item in enumerate(items):
                item_path = os.path.join(path, item)
                if i < len(items) - 1:
                    tree.append(indent + '├── ' + item)
                    if os.path.isdir(item_path):
                        list_directory_tree(item_path, indent + '│   ')
                else:
                    tree.append(indent + '└── ' + item)
                    if os.path.isdir(item_path):
                        list_directory_tree(item_path, indent + '    ')

        list_directory_tree(self.dir_path)

        return "\n".join(tree)




from agency_swarm import BaseTool


class CheckCurrentDir(BaseTool):
    """
    This tool checks the current directory.
    """

    def run(self):
        import os

        if not self.shared_state.get('app_directory'):
            return "You must create an app first to use this tool."

        return os.getcwd()



import base64
import os
import signal
import subprocess

import agency_swarm.agents.BrowsingAgent.tools.util.selenium
from agency_swarm.tools import BaseTool
from pydantic import Field

# agency_swarm.agents.BrowsingAgent.tools.util.selenium.selenium_config = {
#     "chrome_profile_path": None,
#     "headless": True,
#     "full_page_screenshot": True,
# }

from agency_swarm.agents.BrowsingAgent.tools.util import get_web_driver, get_b64_screenshot
from agency_swarm.util import get_openai_client


class AnalyzeContent(BaseTool):
    """
    This tool analyzes the current website content developed by the Web Developer agent.
    By asking questions you can ensure that the current website matches your requirements.
    You can only use this tool after the web developer agent has developed the website.
    """
    question: str = Field(
        ..., description="Question to ask about the contents of the current webpage."
    )

    def run(self):
        wd = get_web_driver()

        # make sure to run the web dev server first
        wd.get("http://localhost:3000")

        wd = get_web_driver()

        client = get_openai_client()

        screenshot = get_b64_screenshot(wd)

        # save screenshot locally
        # with open("screenshot.png", "wb") as fh:
        #     fh.write(base64.b64decode(screenshot))

        messages = [
            {
                "role": "system",
                "content": "Your primary task is to accurately extract and provide information in response to user queries based on webpage screenshots. ",
            },
            {
                "role": "user",
                "content": [
                    {
                        "type": "image_url",
                        "image_url": f"data:image/jpeg;base64,{screenshot}",
                    },
                    {
                        "type": "text",
                        "text": f"{self.question}",
                    }
                ]
            }

        ]

        response = client.chat.completions.create(
            model="gpt-4o",
            messages=messages,
            max_tokens=1024,
        )

        message = response.choices[0].message
        message_text = message.content

        return message_text
